#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "hashmap.h"

#define DEBUG 1
#if DEBUG
#define DEBUG_PRINTF(...) printf(__VA_ARGS__)
#else
#define DEBUG_PRINTF(...) (void)0
#endif

/* computes the hash of a key */
uint32_t hash(K key) {
  /* Please do not modify this function */
  uint32_t ret = 0;
  for(int i=0; i<STRING_LEN; i++) {
    ret = ret ^ (key[i] << (i%4));
  }

  return ret % ARRAY_SIZE;
}

/* initialize a hashmap */
void hashmap_init(struct hashmap_t *hashmap) {
  /* initialize the hashmap entries */
  for(int i=0; i<ARRAY_SIZE; i++)
    hashmap->entries[i] = NULL;
}

/* destroy a hashmap */
void hashmap_finalize(struct hashmap_t *hashmap) {
  /* free all the allocated entries */
  for(int i=0; i<ARRAY_SIZE; i++) {
    struct hashmap_entry_t *cur = hashmap->entries[i];
    while(cur) { 
      struct hashmap_entry_t *next = cur->next;
      free(cur); 
      cur = next;
    }
  }
}

/* return the hashmap entry that corresponds to key */
static struct hashmap_entry_t * get(struct hashmap_t *hashmap, K key) {
  uint32_t h = hash(key);
  struct hashmap_entry_t *list = hashmap->entries[h];
  while(list) { 
    if(strcmp(list->key, key) == 0) 
      return list;
    list = list->next;
  }

  return NULL;
}

/* insert (key, value) into the hashmap */
void hashmap_put(struct hashmap_t *hashmap, K key, V value) {
  /* To simplify the implementation, we assume that key is not already
     in the hashmap */
  uint32_t h = hash(key);
  DEBUG_PRINTF("Putting (%s, %s) in %u\n", key, value, h);

  struct hashmap_entry_t* new_entry = malloc(sizeof(struct hashmap_entry_t));
  strncpy(new_entry->value, value, STRING_LEN);
  strncpy(new_entry->key, key, STRING_LEN);
  new_entry->hash = h;

  new_entry->next = hashmap->entries[h];
  hashmap->entries[h] = new_entry;    
}

/* get the value that corresponds to key.
 * return 1 if the key is found, or 0 otherwise
 */
int hashmap_get(struct hashmap_t *hashmap, K key, V value) {
  uint32_t h = hash(key);
  DEBUG_PRINTF("Looking for %s in %u\n", key, h);

  /* PLEASE DO NOT MODIFY THIS FUNCTION */  
  struct hashmap_entry_t *cur_node = get(hashmap, key);
  if(cur_node) {
    DEBUG_PRINTF("\tfound %s\n", cur_node->value);
    strncpy(value, cur_node->value, STRING_LEN);
    return 1;
  }
  
  DEBUG_PRINTF("\tnot found\n");
  return 0;
}

